package org.apache.bcel.generic;

import org.apache.bcel.Const;
import org.apache.bcel.util.ByteSequence;
import org.apache.bcel.ExceptionConst;
import java.io.IOException;
import java.io.DataOutputStream;

public class NEWARRAY extends Instruction implements AllocationInstruction, ExceptionThrower, StackProducer
{
    private byte type;
    
    NEWARRAY() {
    }
    
    public NEWARRAY(final BasicType type) {
        this(type.getType());
    }
    
    public NEWARRAY(final byte type) {
        super((short)188, (short)2);
        this.type = type;
    }
    
    @Override
    public void accept(final Visitor v) {
        v.visitAllocationInstruction(this);
        v.visitExceptionThrower(this);
        v.visitStackProducer(this);
        v.visitNEWARRAY(this);
    }
    
    @Override
    public void dump(final DataOutputStream out) throws IOException {
        out.writeByte(super.getOpcode());
        out.writeByte(this.type);
    }
    
    @Override
    public Class<?>[] getExceptions() {
        return new Class[] { ExceptionConst.NEGATIVE_ARRAY_SIZE_EXCEPTION };
    }
    
    public final Type getType() {
        return new ArrayType(BasicType.getType(this.type), 1);
    }
    
    public final byte getTypecode() {
        return this.type;
    }
    
    @Override
    protected void initFromFile(final ByteSequence bytes, final boolean wide) throws IOException {
        this.type = bytes.readByte();
        super.setLength(2);
    }
    
    @Override
    public String toString(final boolean verbose) {
        return super.toString(verbose) + " " + Const.getTypeName(this.type);
    }
}
